"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cryptrack = void 0;
const elliptic_1 = require("elliptic");
const crypto_js_1 = require("crypto-js");
class Cryptrack {
    constructor() {
        this._curve = new elliptic_1.ec('secp256k1');
        this._keys = this._curve.genKeyPair();
        this._store = new Map();
    }
    /**
     * Регистрирует сервис, вычисляет sharedKey и сохраняет его в памяти
     */
    register(id, publicKeyHex) {
        try {
            const peerPubKey = this._curve.keyFromPublic(publicKeyHex, 'hex').getPublic();
            const sharedSecret = this._keys.derive(peerPubKey).toString(16);
            const sharedKey = (0, crypto_js_1.SHA256)(sharedSecret).toString();
            this._store.set(id, sharedKey);
            return sharedKey;
        }
        catch (err) {
            throw new Error(`Invalid public key: ${err.message}`);
        }
    }
    /**
     * Возвращает sharedKey по id (если был ранее зарегистрирован)
     */
    getKey(id) {
        return this._store.get(id);
    }
    /**
     * Шифрует строку с использованием sharedKey по id
     */
    encrypt(id, data) {
        const key = this.getKey(id);
        if (!key)
            throw new Error(`Shared key for ID "${id}" not found`);
        return crypto_js_1.AES.encrypt(data, key).toString();
    }
    /**
     * Расшифровывает строку с использованием sharedKey по id
     */
    decrypt(id, encrypted) {
        const key = this.getKey(id);
        if (!key)
            throw new Error(`Shared key for ID "${id}" not found`);
        try {
            const bytes = crypto_js_1.AES.decrypt(encrypted, key);
            const decrypted = bytes.toString(crypto_js_1.enc.Utf8);
            if (!decrypted)
                throw new Error('Failed to decode decrypted data');
            return decrypted;
        }
        catch (err) {
            throw new Error(`Decryption failed: ${err.message}`);
        }
    }
    /**
     * Возвращает публичный ключ текущего Bilibo
     */
    getPublicKey() {
        return this._keys.getPublic('hex');
    }
}
exports.Cryptrack = Cryptrack;
